
from azee.util.geom import vec3, quat

from azee.spec import skel_bones, linked_bone, root_bone, parent_bone


class Posture:
    """
    This class represents an instant configuration of the avatar.
    """
    def dup(self):
        """
        returns a copy of the posture (one that can be touched with no effect on this object)
        """
        return NotImplemented

    def jointRot(self, bone: str) -> quat:
        """
        returns the local rotation of the argument bone in the parent coordinate system
        """
        return NotImplemented

    def jointPos(self, bone) -> vec3:
        """
        returns the position of the origin of the argument bone in the world coordinate system
        """
        return NotImplemented

    def sitePos(self, site: str) -> vec3:
        """
        returns the position of the given body site in the world coordinates
        """
        return NotImplemented

    def MsspPos(self) -> vec3:
        return NotImplemented

    def addresseePos(self) -> vec3:
        return NotImplemented

    def upVect(self) -> vec3:
        return NotImplemented

    def dirVect(self, bone: str) -> vec3:
        return NotImplemented

    def nrmVect(self, bone: str) -> vec3:
        return NotImplemented

    def MsspPos(self) -> vec3:
        return NotImplemented

    def addresseePos(self) -> vec3:
        return NotImplemented



class DummyPosture(Posture):
    def __init__(self):
        self.rots = {b: quat(vec3(0, 0, 1), 0) for b in skel_bones()}

    def dup(self):
        res = DummyPosture()
        for b, rot in self.rots.items():
            res.setRotation(b, rot)
        return res

    def jointRot(self, bone: str) -> quat:
        return quat(vec3(1, 0, 0), 1)

    def jointPos(self, bone) -> vec3:
        i = 0
        while bone != root_bone():
            i += 1
            bone = parent_bone(bone)
        return vec3(0, 0, i)  # dummy value

    def sitePos(self, site: str) -> vec3:
        return self.jointPos(linked_bone(site)) + vec3(.5, .5, .5)  # dummy value

    def MsspPos(self) -> vec3:
        return vec3(0, 0, 0)  # dummy value

    def addresseePos(self) -> vec3:
        return vec3(0, 10, 1)  # dummy value

    def upVect(self) -> vec3:
        return vec3(0, 0, 1)  # dummy value

    def dirVect(self, bone: str) -> vec3:
        return vec3(1, 0, 0)  # dummy value

    def nrmVect(self, bone: str) -> vec3:
        return vec3(0, 1, 0)  # dummy value

    def setRotation(self, b, rot):
        self.rots[b] = rot

dummy_posture = DummyPosture()
