
from io import StringIO


def xml_tagstr(elt, attr_lst):
	return elt + ''.join([' %s="%s"' % attr_val for attr_val in attr_lst])


class XML_output(StringIO):
	def __init__(self, indent='\t', lvl=0, header=''):
		'''
		header is a string of lines to prepend in output
		'''
		super().__init__()
		self.tagstack = []
		self.indentation_str = indent
		self.indentation_level = lvl
		for line in header:
			self.indent_text(line)

	def indentln(self, txt):
		self.write((self.indentation_level + len(self.tagstack)) * self.indentation_str)
		self.write(txt)
		self.write('\n')

	def indent_text(self, txt):
		for line in txt.split('\n'):
			self.indentln(line)

	def line_comment(self, msg):
		self.indentln('<!-- %s -->' % msg)

	def block_comment(self, lines):
		'''
		lines can be a list of anything, elements will be printed line by line
		if it is a string, each '\n'-separated line is indented as if passed in a list
		'''
		if isinstance(lines, str):
			lines = lines.split('\n')
		self.indentln('<!--')
		for line in lines:
			self.indentln('  %s' % line)
		self.indentln('-->')

	def line_tag(self, element, attr=[]):
		self.indentln('<%s/>' % xml_tagstr(element, attr))

	def open_tag(self, element, attr=[]):
		self.indentln('<%s>' % xml_tagstr(element, attr))
		self.tagstack.append(element)

	def close_tag(self, cktag=None):
		cltag = self.tagstack.pop()
		if cktag:
			assert cktag == cltag, 'Not closing expected XML tag %s' % cltag
		self.indentln('</%s>' % cltag)



